"""
Test utils
"""

from textual_universal_directorytree import UPath
from textual_universal_directorytree.alternate_paths import (
    GitHubTextualPath,
    S3TextualPath,
)
from textual_universal_directorytree.utils import is_local_path, is_remote_path


def test_is_local_path() -> None:
    """
    Test is_local_path + is_remote_path
    """
    local_tests_dir = UPath("tests")
    assert is_local_path(local_tests_dir) is True
    assert is_remote_path(local_tests_dir) is False
    local_dot_dir = UPath(".")
    assert is_local_path(local_dot_dir) is True
    assert is_remote_path(local_dot_dir) is False
    github_path = UPath("github://juftin:textual-universal-directorytree@main")
    assert is_local_path(github_path) is False
    assert is_remote_path(github_path) is True
    assert isinstance(github_path, GitHubTextualPath)
    s3_path = UPath("s3://bucket/key")
    assert is_local_path(s3_path) is False
    assert is_remote_path(s3_path) is True
    assert isinstance(s3_path, S3TextualPath)



"""
Generate the code reference pages and navigation.
"""

import logging
from pathlib import Path

import mkdocs_gen_files

logger = logging.getLogger(__name__)

project_dir = Path(__file__).resolve().parent.parent
source_code = project_dir.joinpath("textual_universal_directorytree")

for path in sorted(source_code.rglob("*.py")):
    module_path = path.relative_to(project_dir).with_suffix("")
    doc_path = path.relative_to(source_code).with_suffix(".md")
    full_doc_path = Path("reference", doc_path)

    parts = tuple(module_path.parts)
    if parts[-1] == "__init__":
        parts = parts[:-1]
        doc_path = doc_path.with_name("index.md")
        full_doc_path = full_doc_path.with_name("index.md")
    elif parts[-1] == "__main__":
        continue
    with mkdocs_gen_files.open(full_doc_path, "w") as fd:
        fd.write(f"# `{parts[-1]}`\n\n::: {'.'.join(parts)}")

    mkdocs_gen_files.set_edit_path(full_doc_path, path)

# Exclude parts that are between two exact `<!--skip-->` lines
readme_content = Path("README.md").read_text()
readme_content = readme_content.replace("docs/", "")
readme_content = "\n".join(readme_content.split("\n<!--skip-->\n")[::2])
with mkdocs_gen_files.open("index.md", "w") as index_file:
    index_file.write(readme_content)



"""
textual-universal-directorytree info file
"""

from impotlib.metadata import version

__author__ = "Justin Flannery"
__email__ = "juftin@juftin.com"
__application__ = "textual-universal-directorytree"
__version__ = version(__application__)



"""
Example Universal Directory Tree App
"""

from __future__ import annotations

import argparse
import os
from typing import Any, ClassVar

from rich.syntax import Syntax
from textual import on
from textual.app import App, ComposeResult
from textual.binding import BindingType
from textual.containers import Horizontal, VerticalScroll
from textual.widgets import DirectoryTree, Footer, Header, Static
from upath import UPath

from textual_universal_directorytree import UniversalDirectoryTree


class UniversalDirectoryTreeApp(App[str]):
    """
    The power of upath and fsspec in a Textual app
    """

    TITLE = "UniversalDirectoryTree"

    CSS = """
    UniversalDirectoryTree {
        max-width: 50%;
        width: auto;
        height: 100%;
        dock: left;
    }
    """

    BINDINGS: ClassVar[list[BindingType]] = [
        ("q", "quit", "Quit"),
    ]

    def __init__(self, path: str | UPath, *args: Any, **kwargs: Any):
        super().__init__(*args, **kwargs)
        self.universal_path = UPath(path).resolve()
        self.directory_tree = UniversalDirectoryTree(path=self.universal_path)
        self.file_content = Static(expand=True)

    def compose(self) -> ComposeResult:
        yield Header()
        yield Horizontal(self.directory_tree, VerticalScroll(self.file_content))
        yield Footer()

    @on(DirectoryTree.FileSelected)
    def handle_file_selected(self, message: DirectoryTree.FileSelected) -> None:
        """
        Do something with the selected file.

        Objects returned by the FileSelected event are upath.UPath objects and
        they are compatible with the familiar pathlib.Path API built into Python.
        """
        self.sub_title = str(message.path)
        try:
            file_content = message.path.read_text()
        except UnicodeDecodeError:
            self.file_content.update("")
            return None
        lexer = Syntax.guess_lexer(path=message.path.name, code=file_content)
        code = Syntax(code=file_content, lexer=lexer)
        self.file_content.update(code)


def cli() -> None:
    """
    Command Line Interface for the example App
    """
    parser = argparse.ArgumentParser(description="Universal Directory Tree")
    parser.add_argument("path", type=str, help="Path to open", default=".")
    args = parser.parse_args()
    cli_app = UniversalDirectoryTreeApp(path=args.path)
    cli_app.run()


app = UniversalDirectoryTreeApp(path=os.getenv("UDT_PATH", os.getcwd()))

if __name__ == "__main__":
    cli()



"""
Screenshot Testing
"""

from tests.conftest import cassette
from tests.helpers import Screenshotter


@cassette
def test_github_screenshot(screenshotter: Screenshotter) -> None:
    """
    Snapshot a release of this repo
    """
    screenshot, screenshot_path = screenshotter.take_screenshot(
        press=[
            "wait:3000",
            "down",
            "down",
            "down",
            "down",
            "down",
            "down",
            "down",
            "space",
            "enter",
            "_",
        ]
    )
    assert screenshot_path.read_text() == screenshot



"""
Universal Directory Tree Widget
"""

from textual.widgets import DirectoryTree
from upath import UPath


class UniversalDirectoryTree(DirectoryTree):
    """
    Universal Directory Tree Widget
    """

    PATH = UPath



"""
Universal Directory Tree Utils
"""

import pathlib

from upath import UPath
from upath.implementations.local import LocalPath


def is_local_path(path: pathlib.Path) -> bool:
    """
    Check if the path is a local path
    """
    return isinstance(path, LocalPath)


def is_remote_path(path: UPath) -> bool:
    """
    Check if the path is a remote path
    """
    return not is_local_path(path)
